;**********************************************************************
; MultiMcu/CMOS/TTL Development & Breakout board sample ASM code.
;Developed by Ancel Bhagwandeen, mosaicmerc@yahoo.com
; May 2014.
;featured on ClubJameco.com
;Demos 7 Segment x 4 digit, tactile button, LED, Beep and Rotary encoder functions.
;Note the Demo jumper wire schematic diagram required for this to work.
;**********************************************************************




	errorlevel -302 ; suppress message 302 from list file

	list		p=16f690		; list directive to define processor
	#include	"P16F690.inc"		; processor specific variable definitions
;	__IDLOCS   0x0009  ;Your Firmware Version if you care

	;radix D
 	;__config 0x33c4 ; use this for a forced config word.

	
	__CONFIG  _IESO_OFF & _FCMEN_OFF & _PWRTE_ON  & _WDT_OFF& _INTOSCIO&  _CP_OFF&_CPD_OFF& _MCLRE_OFF  
	
	; Tip: Method of Voltage sampling using a 1:3.2 voltage divider made of 3 x 10K & 2 x 1K resistors. This brings the max sample v to 3.2 x 5 = 16V.
		; ADC Hardware sample @ 5Vref , 10 bits= V/3.2 x 1024/5         , then,  x 25/16 gives the result. So we sample 25 times and divide by 16 for precision.
	
	; Tip: Method of Voltage sampling using no voltage divider for a value < 5V . 
		; ; ADC Hardware sample @ 5Vref , 10 bits= V/1  x 1024/5  , then divide sample  by 2 , sample 125 times then divide by 128 = 2 decimal place result.
	

	;****************GPR declarations****************		; note Bank0 last 16bytes,0x70 to 0x7f, visible in all banks as local last 16.
	
	Secondcount		EQU	0x20
	TmpBnk0			EQU	0x20; reuseable after memfill init.
	PORTAShadow		EQU	0x21; used to update port pins and avoid read,modify,write issues.
	Timebase			EQU	0x22
	Count_8ms		EQU	0x23
	Dbouncecount		EQU	0x24
	Voffset			EQU	0x25
		FlagA		EQU	0x26
			#define tick			FlagA,0;1 sec tick
			#define	beep		FlagA,1; for beep tasking.
			#define seg_test		FlagA,2; flag to test 7 segment display.
			#define setdecimal	FlagA,3; flag a single displayed decimal place.
			#define adc_done		FlagA,4; set when total adc sample is rdy as oversampling is used.
				
	
	Count_500usec	EQU	0x27; track 1/2ms  interrrupt count.		
	
	;-------------------------------6 bytes used for Debouncing up to 8 switches-------------------------------;
	Sw_state 			Equ	0x28
	KeyTrans			EQU	0x29 
	HoldTrans		EQU	0x2A
	LastKeystate		EQU	0x2B
	DbKeystate		EQU	0x2C
	HoldKeystate		EQU	0x2D
	Count_8ms_old	EQU	0x2E
	Digitstrobe		Equ	0x2F
	Offset			EQU	0x30
	
	;These next four GPR must be kept in this sequence as they are referred to by indiect addressing for the Strobe display code.
	Units			EQU	0x31
	Tens				EQU	0x32
	Hundreds			EQU	0x33; req'd for a 4 digit display
	Thousands		EQU	0x34; req'd for a 4 digit display
	;_____________________________________________________________________________________________
	TempL			Equ	0x35; Reusable temporary GPRs, usually used to feed the math functions and binary to dec function.
	TempH			Equ	0x36; Reusable temporary GPRs, usually used to feed the math functions and binary to dec function.
	VoltLo			Equ	0x37; batt voltage sample
	VoltHi			Equ	0x38
	LastSample		Equ 	0x39 ;
	VoltADC_count	Equ	0x3a ; # of adc samples - oversamples
	FixedL			Equ	0x3b; math routine GPR
	FixedH			Equ	0x3c; math routine GPR
	Samplecount		Equ	0x3d; track # of voltoffset samples taken
	R_Dbstate		Equ	0x3e; rotary debounce state byte
	R_dbcount		Equ	0x3f; tracking debounce timer

	; Multibank storage for easy GPR access.	; note Bank0 last 16bytes,0x70 to 0x7f, visible in all banks as local last 16.
	WregTmp			EQU 	0x70
	STATmp			EQU	0x71
	temp_pclath		EQU	0x72
	Tmp				EQU	0x73; gen purpose local gpr, used internally within each code segment.
	statechange		equ	0x74


	
	#Define 	KEY0	PORTA,3; pin goes low when key is pressed. 
	#define 	Unitsdriver	PORTB,7
	#define 	Tensdriver	PORTB,6
	#define 	Hundredsdriver	PORTB,5;
	#define		Thousandsdriver	PORTB,4 ; 
	#define 	DIGITS	4 ; make this 2 for a 2digit optional display.
	#define 	FOSC 		D'2000' ; Oscillator Clock in kHz	

	
		;******** Prog Start ********;
	Org	0x000	; processor reset vector
 	goto	Main	; go to beginning of program init.
	Org	0x04	; interrupt vector
	
;Preserve key registers first!
	
	movwf WregTmp; save wreg in an allbanks GPR
	movf STATUS,w
	clrf STATUS; sets bank0
	movwf STATmp; save status
	movf  PCLATH,w     ; save PCLATH as well
	movwf temp_pclath
	clrf STATUS; BANK 0
   	clrf PCLATH       ; now on code page 0 for sure too.

;identify which ISR (if more than 1 used) 
	btfss PIR1,TMR2IF; else
	goto Regrestore
		
		
;*********timer code**********

	incf Count_500usec,f; 1/2 ms counter, to allow for a 2Khz beep frequency.
	incf R_dbcount,f;counts 1/2 ms intervals for debouncing  encoder.
	
	btfss Count_500usec,4; skip every 16 periods = 8 ms.
	goto Assessbeep
	clrf Count_500usec; reset.
	incf Count_8ms,f; general 8ms counter .
	incf Timebase,f; used as foundation timer for seconds etc.
	incf Dbouncecount,f ;counts 8ms intervals for debouncing switches
	movf Timebase,w
	sublw .125 ; 125 x 8ms = 1 sec
	skpz
	goto Assessbeep
	bsf tick; 1sec flag
	btfsc beep
	bcf beep; clr the beep flag.
	bcf seg_test; clr the 7 seg test flag
	bcf PORTAShadow,4; clr the  LED
	bcf PORTAShadow,5 ;and the spkr pin
	incf Secondcount,f;  count 1 per sec.
	
	
	
Assessbeep;
	btfss beep
	goto ClrIRQ
	movlw b'00110000'
	xorwf PORTAShadow,f ; toggle beep pulse (A5) and LED (A4)
	bcf setdecimal; cancel decimal place for 'beep' text display on 7 segment.
	
ClrIRQ
	bcf PIR1,TMR2IF
Regrestore; Restore registers
	movf temp_pclath,w ;get PCLATH
	clrf STATUS; bank0
  	movwf PCLATH  ;Move W into PCLATH
	movf STATmp,w
	movwf STATUS ; restore status
	swapf WregTmp,f ; swap nybbles
	swapf WregTmp,w ; swap again and leave in wreg.
	retfie ;return & reactivate GIE.

Dpattern	addwf PCL,f ; offset prog counter to lookup which 7 segment # to be displayed
	retlw 0x3f ; code for 0, bit 7 is the dot. Common cathode. Invert values then clr bit 7 (dot) for common anode.
	retlw 0x06; 1
	retlw 0x5b; 2
	retlw 0x4f ;3
	retlw 0x66 ;4
	retlw 0x6d ;5
	retlw 0x7d ;6
	retlw 0x07 ;7
	retlw 0x7f ;8
	retlw 0x67 ;9
	;retlw 0x77 ;A
	;retlw 0x7c ;b
	;retlw 0x39 ;C
	;retlw 0x5e ;d
	;retlw 0x79 ;E
	;retlw 0x71 ;F
	;retlw 0x3D ;G
	;retlw 0x74 ;h ,
	;retlw 0x76; is 'H' ,
	;retlw 0x38; is 'L' , 
	;retlw 0x78 ;= t
	;retlw 0x10 ;i
	;retlw 0x50 ;r
	;retlw 0x54 ;n
	;retlw 0x73 ;P
	;retlw 0x3e ;U
	
	
	
Main
	banksel OSCCON
	movlw b'01010001'
	movwf OSCCON ; set intosc 2Mhz, Osc setup based on config fuses . (2Mhz for pwr saving, 2uS per instruction)
	clrf TRISA; all dig. outs
	clrf TRISC; all dig. outs
	clrf TRISB; all dig. outs
	bsf TRISA,3;pushbutton - dig. input.
	bsf TRISA,2; dig. input - will use for analog input sample.
	bsf TRISA,1; rotary encoder
	bsf TRISA,0;rotary encoder
		
	movlw .249; PR2 comparator for TMR2;  250 steps , 0 to 249.
	movwf PR2
	bsf PIE1,TMR2IE; enable TMR2IF
	banksel ADCON1
	movlw b'00010000'
	movwf ADCON1; Fosc/8 TAD = 4uSec. Note pages 111 and 116 in specsheet pdf.
		
		
	banksel ANSEL
	clrf ANSEL; all digital.
	bsf ANSEL,2; Ra2,an2 is analog in - batt V
		
	;TMR2 setup for 1/2ms irq, good for a 2Khz beep.
	clrf STATUS
	movlw b'00000100'; 1 post scale, 1 prescale = 1 scaledown
	movwf T2CON; Fosc/4 /(PR2 *Postscle)== (2000000/4)/(250 * 1) = 2000Hz irq., 1/2ms period.
	
	bsf INTCON,7 ;  general interrupts enable
	bsf INTCON,6; Peripheral Int enable - start Tmr1/ccp1/Tmr2 ISR, if enabled in PIE1 register .
		
	
	GPRinit; Auto clr all GPR on boot.
	clrf STATUS
	movlw 0x21; bank 0
	movwf FSR	
	movlw .94; # of bytes to fill.
	movwf TmpBnk0; this is used as a temp on startup.
	call Memfill
	


	movlw 0xA0; bank1
	movwf FSR	
	movlw .79
	movwf TmpBnk0
	call Memfill
	

	bsf STATUS,IRP; bank sel for FSR to bank 2/3
	movlw LOW (0x120); bank2
	movwf FSR	
	movlw .79
	movwf TmpBnk0
	call Memfill
		
		
	clrf STATUS
	clrf PORTA ; 
	clrf PORTC
	clrf PORTB
	
	;______A few init values______
	movlw .125
	movwf VoltADC_count ;init value
	movwf Voffset
	bcf adc_done;init
	goto Mainloop
	
 Memfill; init GPR blocks with zero.
	decfsz TmpBnk0,f; skip when TmpBnk0 has counted down to zero.
	goto Erase
	return
Erase
	clrf INDF; clr ram cell
	incf FSR,f; inc pointer
	goto Memfill
	
	
	
Mainloop; manage all subroutine calls.
	movf PORTAShadow,w; buffer
	movwf PORTA; avoid, read write modify, probs.
	call Keypad; sample keypress
	call Strobe ; display multiplexed digit swap
	call SampleADCs; voltages
	call EvalSamples; integer arithmetic and decimal outputs
	call ScanEncoderPort; Rotary encoder.
	call ApplyEncoder; apply encoder output
	call Evalkey ; decide what to do if a key pressed.
	goto Mainloop
	
ApplyEncoder;; apply encoder output
	addwf Voffset,w
	skpz
	movwf Voffset ; Causes a +/- ,127 bit change in the displayed 7-segment reference voltage
	return

ScanEncoderPort	;_________ Eg. Bourns PEC11 series_________________________________________________________________________________________________________
				; Quadrature (full or detent) rotary encoder signals (A=RA0,B=RA1) , debounced. R_dbcount is an interrupt increment variable, used to tgt a >4ms debounce period.
				; If rapidly turning the encoder causes a lack of response, try reducing the debounce time a bit. Use 10 - 47K weak pullups on pulse lines.
				; R_Dbstate tracks the valid debounced condition of the encoder bits RA0,1.
				; If the encoder bits cannot be placed on the matching portA bits as coded here, read the bits & map them into a RAM byte  (bits 0,1) and replace the below PORTA read with the RAM byte read.
				; Returns W-register = 0 for no change, .255 for CCW and 1 for CW changes per pulse. Simple to add W-reg to any integer variables  afterward.
				;______________________________________________________________________________________________________________________________
		movf PORTA,w		; replace with the mapped RAM byte as mentioned if required..
		andlw d'3'         	 	; filter bit 0,1 for a change. 
		xorwf LastSample,w  	; set changed bits
		xorwf LastSample,f  	; lastsample=currentsample. Affects zero flag.
		andlw d'3'          		; w-reg filter bit 0,1 for a change. Refresh zero flag
		skpz
		clrf R_dbcount			; start the debounce countdown.
		btfss R_dbcount,2		;Debounce  period in this example. Depends on particular interrupt speed. (suggest  this:  >4 ms by choosing an appropriate R_dbcount bit)
		retlw 0
		;______________________________________________________________________________________________________________________________
		; debounce time has passed. Processing validated changes.
		;______________________________________________________________________________________________________________________________
		movf LastSample,w	; fetch the recent encoder bits sample.
		xorwf R_Dbstate,w	; get changes bet. current state and debounced state.
		xorwf R_Dbstate,f		; apply them to DB state
		andlw d'3'         		; w-reg filter bit 0,1 for any change.
		skpnz
		retlw 0				; no debounced change.
		;______________________________________________________________________________________________________________________________
		;Else Test each Encoder bit to determine change direction.
		;______________________________________________________________________________________________________________________________
		clrc
		andlw d'2'         		 ; filter bit 1
		skpnz
		;---------------------------------------------------------------------------------------------------------------------------------------------------
		;********************QUADRATURE MODE CODE:  No Detent (full) or Detent mode (1/4)
		;---------------------------------------------------------------------------------------------------------------------------------------------------
		;goto Testbit0		; ************* Enable this line for full quadrature, AND ALSO,
				;--------comment out this block for full quadrature--------------
		retlw 0
		movf R_Dbstate,w; 
		andlw d'2';
		skpnz;
		retlw 0
				;---------------block end -----------------
		
		rlf R_Dbstate,w       	 ; align encoder debounced bits 
		xorwf R_Dbstate,w  	; compare bits
		andlw d'2'         		 ; filter bit 1
		skpnz
		retlw 1         			; 1=> 1 clockwise pulse
		retlw d'255'                	; 255=> 1 anticlockwise pulse.
	Testbit0
		rrf R_Dbstate,w         	; align encoder debounced bits 
		xorwf R_Dbstate,w  	; compare bits
		andlw d'1'          		; filter bit 0
		skpz
		retlw 1         			; 1=> 1 clockwise pulse
		retlw d'255'               	 ; 255=> 1 anticlockwise pulse.
		;______________________________________________________________________________________________________________________________
 
	

	
Evalkey; decide what to do if a key pressed.
	btfss  KeyTrans,0; test key0 for short press.
	goto Testlongpress
	bsf beep; task a beep.
	clrf  Timebase; ensure beep lasts 1 second
	;display the text 'Beep'
	movlw 0x7c; 'b'
	movwf Thousands
	movlw 0x79; 'E'
	movwf Hundreds
	movwf Tens
	movlw 0x73;p
	movwf Units
	return
Testlongpress
	btfsc HoldTrans,0; test key 0 for longpress.
	bsf seg_test ; test all display segments; reset in interrupt routine every 1 sec.
	return
	
EvalSamples; process adc results into usable figures
	btfss adc_done
	return
	movlw .125
	movwf  VoltADC_count; reset # of samples
	bcf adc_done
	clrf TempH
	
	; VoltL/H oversample divide by 128 to obtain  a 2 dec. point value.
	rlf VoltLo,w ; sets the carry flag as required.
	rlf VoltHi,w; x 2
	; now take only the hi byte = V x2/256 = V/128
	
	movwf TempL ; varable used to display decimal values in Bindec routine.
	skpnc
	bsf TempH,0; handle any overflow to the hi byte.
	; add the rotary encoder input to the displayed value.
	movf Voffset,w
	addwf TempL,f
	skpnc
	incf TempH,f
	clrf VoltLo
	clrf VoltHi
	bsf setdecimal
	goto Bindec; display voltage. on 7 segment Display.
	
	
	
SampleADCs; sample voltages via analog inputs.
	btfsc adc_done
	return
	movlw b'10001001'; enable ADC, right justified and select AN2- battVoltage
	movwf ADCON0
	btfsc ADCON0,GO
	goto $-1; wait for sampling.
Getresult; capture last 10bit sample 
	banksel ADRESL
	movf ADRESL,w
	banksel TempL
	movwf  TempL
	movf ADRESH,w
	movwf TempH
	; now divide by 2 to keep the 125, 10bit samples within a 2 byte integer limit of 65535.
	clrc
	rrf TempH,f
	rrf TempL,w
	
	;Sum to cumulative sample.
	addwf VoltLo,f
	skpnc
	incf TempH,f
	movf TempH,w
	addwf VoltHi,f
	
	decf VoltADC_count,w; count down the # of samples req'd
	addlw 0
	skpnz
	goto Sampledone
	movwf VoltADC_count
	bsf ADCON0,GO;trigger another sample
	return
Sampledone; all samples taken.
	bsf adc_done; set flag to permit sample evaluation.
	return

	
Strobe	 ; this routine strobes the 2/4 digit 7 segment display. Parallel transfer. Possible to use with a 74HC595 serial  transfer with code modifications.
		; Search & invert all  'D_invert' commented code lines should the common (cathode) drive line logic need to be inverted to common anode.
		; since each LED segment consumes the rated LED power (eg. 20mA continuous or 50mA pulsed in <1mS), Max common mode drain (incl. decimal) = 8x rated, eg. 160mA
		; This loading requires a common mode driver transistor with resultant driver line logic inversion.
		; each LED segment dropping resistor for a 10mA, 2 Vf RED display @5V supply=> 5-2=3V, V=IR => R=V/I => 3/.01 = 300 ohms.
		; @ 1/2  or 1/4 duty cycle overdriving with a 220ohm is reasonable,.
	
	movf Count_8ms,w; get newvalue
	xorwf Count_8ms_old ,w; compare to old value
	xorwf Count_8ms_old ,f; old= new
	andlw b'00000001';  test for 8ms change ( each digit display time, 4 digit = 32ms total refresh rate).
	skpnz ; skip if change , else
	return
	
	incf Digitstrobe,f ; increment the display digit we are updating.
	;Next, make all common drive lines 'disabled' hi for CC or Low for CA. The assumed driver transistor on the common lines inverts this logic! This permits setup of each digit's pattern data without display flicker.
	;________________________________________________________
	bcf Thousandsdriver ;  *** D_invert
	bcf Hundredsdriver; *** D_invert
	bcf Tensdriver;  *** D_invert
	bcf Unitsdriver;  *** D_invert
	;________________________________________________________
	clrf Tmp ; clear bitmask byte - local temp variable , used to sequentially track which digit to strobe.
	movf Digitstrobe,w ; load counter - informs which digit to strobe.
	andlw b'00000011' ; checking for bits 1 & 0 only for counting  (up to 4) digits 0,1,2,3.  Modify this to '00000111'  and add sequential GPR display buffers if u need to go to 8 digits.
	movwf Offset; local digit select counter.
	incf Offset,f; correct for next decf cmd.
;'bbbbXXXX' bitmask will be used track digit drive pins. (Buffer 4,3,2,1 : thou,hund,tens,units)
	movlw Thousands ; highest digit value, display buffer. GPR for the digits must be in a continuous ascending order sequence.
	movwf FSR ; setup indirect addressing for Display Buffers.
	
	;Next 10 lines configures for a 2 digit display or a 4 digit. The defines above determine  how this works
	; For a 2 digit display we simply repeat the lower 2 digits so the 4 digit code flow remains intact.
	movlw DIGITS; a constant to toggle 2 or 4 digits display
	sublw 4
	 ;custom code forcing a decimal point as an indicator. Can be deleted from regular display code.
	 btfsc setdecimal
	 bsf Hundreds,7; Hundreds Digit 
	 btfss setdecimal
	 bcf  Hundreds,7
	;____________________Removable force decimal code ends_______________________________________
	skpnz; if 2 digits, else
	goto Chk_Dcount ; 
	 movf Tens,w
	 movwf Thousands
	 movf Units,w
	 movwf Hundreds
	 
	
Chk_Dcount
	bsf Tmp,4 ; initialize the bitmask to 0001XXXX
	bcf STATUS,C; clr C
	decfsz Offset,f
	goto ShiftDigit
	
	movf INDF,w ; get correct buffer. Always complement this pattern for CA. => comf INDF,w ; ****
	btfsc seg_test ; set in keypress handler code.
	movlw .255;testing all display segments
	movwf PORTC ;  apply 7 seg. pattern to display.

	; Code below selects which common cathode drive line to strobe.
	;Next 4 lines configures for a 2 digit display or a 4 digit. The defines above determine  how this works
	; For a 2 digit display we simply repeat the lower 2 digits so the 4 digit code flow remains intact.
	
	movlw DIGITS
	sublw 4
	skpz
	goto Twodigits
			
	btfsc Tmp,4 
	bsf Thousandsdriver ;   A pnp (CA) or npn (CC) transistor common driver is assumed (inverts logic) *** D_invert
	btfsc Tmp,5
	bsf Hundredsdriver;  A pnp (CA) or npn (CC) transistor common driver is assumed (inverts logic) *** D_invert
Twodigits;
	btfsc Tmp,6 
	bsf Tensdriver;   A pnp (CA) or npn (CC) transistor common driver is assumed (inverts logic) *** D_invert
	btfsc Tmp,7
	bsf Unitsdriver;   A pnp (CA) or npn (CC) transistor common driver is assumed (inverts logic) *** D_invert
	skpnz
	return
	;handle 2 digit, limited display.
	btfsc Tmp,4 
	goto D_tens
	btfsc Tmp,5
	goto D_units
	return
D_tens;
	bsf Tensdriver;   A pnp (CA) or npn (CC) transistor common driver is assumed (inverts logic) *** D_invert
	movf Tens,w
	movwf PORTC
	return
D_units
	bsf Unitsdriver;   A pnp (CA) or npn (CC) transistor common driver is assumed (inverts logic) *** D_invert
	movf Units,w
	movwf PORTC
	return
	
ShiftDigit; rotate bitmask to select digit to enable.
	rlf Tmp,f
	decf FSR,f ; point to current display buffer
	btfss STATUS,C
	goto Chk_Dcount+2
	goto Chk_Dcount; reinit the mask.


;**********************************

Keypad; Evaluate keypresses for click, hold, open with debounce & transition states. Works in parallel for up to 8 switches. Requires a 8 ms tick (usually done in an ISR) for 'Dbouncecount'
;Setup for Key_0 , in bit 0;  this method allows any pin in any port to be a key /switch as the keys are mapped to a single 8bit byte here. 'Tmp' is a locally valid temp variable for general subroutine use.
; The Transition bytes are set  if a new  debounced keystate just happened . These are cleared 
;Each of the 8 Transition bits in the bytes (Keytrans and HoldTrans) allow for debounced edge triggering events. eg. If the DbKeystate,0  is hi and the Keytrans,0 is high then the debounced key0 press leading edge just happened.  
;If DbKeystate is Hi and Keytrans,0 is lo, then the debounced Keypress is mature.
;If the DbKeystate,0 is lo and the KeyTrans,0 is hi then the debounced key was just released - trailing edge.
; A similar structure applies to HoldTrans and HoldKeystate bytes as applies to a 'long key hold/click' operation - holding down a key for a 1/2 ;second.
; Thus a single key can deliver up to 4 states, leading and trailing edge for both short clicks and long clicks.
; The use of the leading edge transitions is manifest when preventing 'repeating' a routine that responds to the key condition.eg. only respond to a keypress if the transition bit is also set, like make a beep.
;The leading edge can also keep track of which key to respond to if multiple keys are pressed simultaneously.
; eg. The trailing transition can be used for triggering an acknowledgement after the key is released.
; note the KEYDONE code below for testing debounced keypresses easily.

	clrf  Sw_state ;clear switch state byte for all switches evaluation below.
	clrf KeyTrans ;limit all transitions to one complete program loop.
	clrf HoldTrans; " "
	;btfss KEY1 ;  Key0 = pin physical location.  Btfsc used when key is pulled hi when closed.
	;bsf Sw_state,2; 
	btfss KEY0; Key0 = pin physical location. Btfss used when key is pulled low when closed
	bsf  Sw_state,0 ; match  Sw_state bit 0 to  Key state, 1= closed, 0=open.
;Setup for Key_1 , in bit 1
	;btfsc PORTA,6 ;  Key1 pin. = PORTA,6
	;bsf  Sw_state,1 ; match  Sw_state bit 1 to  Key state, 1= closed, 0=open.
	
;Setup for Key_7 , in bit 7
	;btfsc PORTB,6 ;  Key7 pin. = PORTB,6
	;bsf  Sw_state,7 ; match  Sw_state bit 7 to  Key state, 1= closed, 0=open.
		
	
	movf Sw_state,w ; get newstate					eg. Sw0,2 closed	0101
	xorwf LastKeystate,w ; state changes					eg.		1100 =>	1001
	xorwf LastKeystate,f ; make Laststate =Newstate.					1100=>	0101
	andlw b'11111111'; test wreg if any bits changed
	btfss STATUS,Z; skip zero set, else (statechange)
	clrf Dbouncecount; reset counter (counts every 8 msec in ISR)
	btfss Dbouncecount,1 ; skip if bit 1 set=> 16ms passed, else
	goto Testkeyhold;
	movf LastKeystate,w;													0101
	xorwf DbKeystate,w; get changes bet. current state and debounced state.		0110 =>	0011
	movwf KeyTrans; save in transition byte.											0011 (transition bits)
	xorwf DbKeystate,f; apply them to DB state								0110 => 0101					
	
Testkeyhold
	movf  DbKeystate,w	;														0101
	andwf HoldKeystate,w ;zero Hold bits if matching key is released - wreg.				0011 =>	0001
	xorwf HoldKeystate,w ; get Transition from Hold to release							0011 =>	0010
	movwf HoldTrans; update HoldTransition byte.														0010
	xorwf KeyTrans,f ; toggle any matching release from clicks off, release from hold priority.
	xorwf HoldKeystate,f ; now immediately switch off Hold if a key is released - file.		0011 =>	0001
	xorwf DbKeystate,w ; get newly pressed keys to test for hold.(state change)							0101 =>	0100
	btfss STATUS,Z; skip if  no new key pressed, else
	
	
Testforholdcount ; test if timer expired
	btfss Dbouncecount,6 ; 64*8ms = 512ms
	goto Keydone
	movf DbKeystate,w;							0101
	xorwf HoldKeystate,w ; get statechanges			0001 =>	0100
	iorwf HoldTrans,f; update hold transition byte.			0010 =>	0110 (transition bits)
	;using what's still in wreg to apply changes		0100
	xorwf HoldKeystate,f;apply changes				0001 =>	0101
	
Keydone	; simplify testing for keypresses  ***
	movf  DbKeystate,w
	xorlw .255;  toggle to respond to trailing edge of click debounce rather than leading edge. Prevents automatically triggering a shortpress when a long press is desired.
	andwf KeyTrans,f; bits are set to match debounced keys just pressed. Test a key0 short press by Btfss KeyTrans,0 etc.
	movf HoldKeystate,w
	andwf HoldTrans,f; bits are set to match debounced keys just pressed. Test a key0 long press by Btfss HoldTrans,0 etc.
	return;***	

	;___________________________________________________________________________________________________

Bindec	;calculate decimal# for Displaying each 7seg led block, requires  value in TempL/H
	btfsc beep; don't show digits if  beep text active.
	return
	movlw DIGITS ; evaluate # of digits in use to determine Overrange display conditions.
	sublw 2
	skpz
	goto  Fourdigits
	clrf TempH;
	movf TempL,w; incoming low byte display value.
	sublw .99; >99 => overrange for 2 digits.
	skpc
	goto Overrange; force overrange 

Fourdigits;	
	clrf Thousands
	clrf Hundreds
	clrf Tens
	clrf Units
	call Thou ; get thousands in decimal
	movlw .10 ; check for 10,000 or more, for 4 digit overrange conition.
	subwf Thousands,w;  if Thousands < 10, carry is clr.
	btfsc STATUS,C ;
	goto Overrange ;
	Call Hundred ; get hundreds
	call Ten ; get tens & units.
	return
Overrange
	clrf Thousands
	clrf Hundreds
	clrw
	movlw 0x74 ; h pattern
	movwf Tens
	movlw 0x10 ; i pattern
	Movwf Units
	return ; restore this return if custom menuitme code immediately below is unneccessary
	

Thou	movlw .3 ; high byte set to 3 = 768	
	movwf FixedH ; high byte of # to be subtracted.
	movlw .232 ; low byte
	movwf FixedL ; 768 + 232 = 1000's column

ThouCount
	call Subtract
	xorlw .1 ; check for returned wreg value. A  zero occurs if a 1 was returned
	btfss STATUS,Z ; branch if 1 was returned, else
	goto Fixremainder ; due to last subtraction the remainder went -ve, so we fix this.
	incf Thousands,f
	goto ThouCount

Fixremainder ;restore remainder to continue calculating for next decimal column.
	movf    FixedL,w ; low byte
        addwf   TempL,f ; TempL=TempL + FixedL
        movf    FixedH,w ; hi byte
        btfsc   STATUS,C ; check for carry, set if  TempL + FixedL >255
        incfsz  FixedH,w ; Increment due to carry set ,if zero skip next.
        addwf   TempH,f	; TempH=TempH+FixedH (+1 if carry was set)
	return

Hundred	movlw 0 ; high byte set to 0	
	movwf FixedH ; high byte of # to be subtracted.
	movlw .100 ; low byte
	movwf FixedL ; 0 + 100 = 100's column

HdrdCount
	call Subtract
	xorlw .1 ; check for returned wreg value. A  zero occurs if a 1 was returned
	btfss STATUS,Z ; branch if 1 was returned, else
	goto Fixremainder ; due to last subtraction the remainder went -ve, so we fix this.
	incf Hundreds,f
	goto HdrdCount

  
Ten	movlw .10
	subwf TempL,f; test for 10 if carry clr wreg is under 10
	btfsc STATUS,C ;skip if carry clr (-ve result) , else
	incf Tens,f; incr the tens column
	btfsc STATUS,C ; skip if carry clr (-ve result), else
	goto Ten ; now check for 20,30 etc
	movf TempL,w
	addlw .10; as last calc was -ve recreate +ve number.
DispLEDS; 7 segment  pattern lookup calls for each digit.
	call Dpattern
	Movwf Units ; store 7 seg display pattern for units
	movf Tens,w	
	call Dpattern
	movwf Tens; store 7 seg display pattern for tens
	movf Hundreds,w
	call Dpattern
	movwf Hundreds; store 7 seg display pattern for hunds
	movf Thousands,w
	call Dpattern
	movwf Thousands	
	return
	
;Math routines -----------------------------------------------------------------------------------------------------------------------------------------------
	
Subtract; 2byte subtraction, Execute the 2 byte: TEMP-FIXED, result in TEMPL/H, carry flag in wreg.
	movf    FixedL,W
	subwf   TempL,f ; TEMPL= TEMPL-FIXEDL
	movf    FixedH,W
	btfss   STATUS,C; branch if TEMPL>FIXEDL, else
	incfsz   FixedH,W ; setup to reduced TempH by 1 extra (sim. a borrow).
	subwf   TempH,f ;TempH=TempH-FixedH
	btfss	STATUS,C ;branch if TempH>FixedH, else
	retlw .0 ; if carry is clr return with a 0 in wreg
	retlw .1 ; if carry is set return with a 1 in wreg.
	
Div_by_subtract; sequential subtractions until -ve happens, result in Tmp. Divisor in FixedL/H. Dividend in TempL/H
	clrf Tmp
	decf Tmp,f
S_loop;
	incf Tmp,f
	call Subtract
	andlw  1
	skpz
	goto S_loop
	return ; result in Tmp
	
end
	